<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Server.php 3327 2005-06-09 15:11:22Z andrew $
*/

/**
 * XML RPC server for MAX services.
 *
 * @author Demian Turner <demian@phpkitchen.com>
 * @author James Floyd <james@m3.net>
 */

require_once 'XML/RPC/Server.php';
require_once '../../../init.php';

/**
 * Method to return dispatcher map containing information 
 * about defined MAX XML-RPC methods.
 *
 * @return array dispacher map
 *
 * Array
 * (
 *     (
 *         [agencies.getStats] => Array
 *             (
 *                 [function] => Max_RPC_Server_agenciesGetStats
 *                 [signature] => Array
 *                     (
 *                         [0] => Array
 *                             (
 *                                 [0] => boolean
 *                                 [1] => int
 *                                 [2] => struct
 *                                 [3] => struct
 *                             )
 *                     )
 *                 [docstring] => Accepts one int parameter, returns true if agency is successfully deactivated.
 *             )
 *     )
 */
function MAX_RPC_getDispatchMap()
{
    require_once MAX_PATH . '/lib/max/FileScanner.php';
    $map = array();
    
    // scan services folder for server services
    $fileScanner = new Max_FileScanner();
    $fileScanner->addDir(MAX_PATH . '/lib/max/RPC/Server');
    $fileScanner->addFileTypes('php');
    
    // get full path to service files
    $aFileLocations = $fileScanner->getAllFiles();
    
    foreach ($aFileLocations as $fileLocation) {
        require_once $fileLocation;
        
        // formulate expected method name
        $fileName = $fileScanner->getFileName($fileLocation);
        $actionName = substr($fileName, 0, strlen(substr($fileName, 0, strrpos($fileName, '.'))));
        $functionName = 'Max_RPC_Server_' . $actionName;

        if (function_exists($functionName)) {
            $aliasVar = $actionName . '_alias';
            $signatureVar = $actionName . '_sig';
            $docstringVar = $actionName . '_doc';
            
            // check required config values are set
            if (isset(${$aliasVar}) && isset(${$signatureVar}) && isset(${$docstringVar})) {

                // Build map values
                $map[${$aliasVar}] = array(
                    "function"  => $functionName,
                    "signature" => ${$signatureVar},
                    "docstring" => ${$docstringVar},
                    );
            } else {
                echo 'Max_RPC_Server_' . $fileScanner->getFileName($file) . ' required param not defined!!';
            }
        } else {
            echo 'Max_RPC_Server_' . $fileScanner->getFileName($file) . ' method not defined!!';
        }
    }
    return $map;
}

/**
 * Cache wrapper for MAX_RPC_getDispatchMap method.
 *
 * @return array dispacher map
 */
function MAX_RPC_getDispatchMap_cache()
{
    require_once 'Cache/Lite/Function.php';

    $options = array(
        'cacheDir' => MAX_PATH . '/var/cache/',
        'lifeTime' => 0
    );
    
    $cache = new Cache_Lite_Function($options);
    return $cache->call('MAX_RPC_getDispatchMap');
}

$map = MAX_RPC_getDispatchMap_cache();
new XML_RPC_Server($map, $serviceNow = 1, $debug = 0);
?>
